% Copyright (c) 2008, Ludwig Institute for Cancer Research.
% All rights reserved.
% Author: Armand Valsesia <armand.valsesia@licr.org>
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above
%       copyright notice, this list of conditions and the following
%       disclaimer in the documentation and/or other materials
%       provided with the distribution.
%     * Neither the name of the <organization> nor the
%       names of its contributors may be used to endorse or promote
%       products derived from this software without specific prior
%       written permission.
%
% THIS SOFTWARE IS PROVIDED BY GRL ``AS IS'' AND ANY EXPRESS OR
% IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED.  IN NO EVENT SHALL GRL BE LIABLE FOR ANY DIRECT,
% INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
% HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
% STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
% OF THE POSSIBILITY OF SUCH DAMAGE.
%
% function callCNVs
%  - perform loess smoothing (optional)
%  - CNV calling using the Gaussian Mixture Model
% argin :
%   - fn : input file, tab delimited, with header line
%       i.e. chr    pos    sample   sample2 ...
%   - W : loess half-window size, i.e. to do smoothing on 41 probes, set W=20
%   - DO_LOESS : 1 do loess smoothing, else skip (loess smoothed ratios
%   will be exported to a flat file named like inputFile.loess.dat
%   - MaxFunEvals & MaxIter, number of function calls and iterations from
%   the optimization algorithm to shift Gaussian component boundaries
%
% output : a flat file inputFile.cnv.dat, containing the CN dosage value of
% each probe at each sample.
function callCNVs(fn, W,DO_LOESS,MaxFunEvals,MaxIter)

if nargin < 1
    error('Missing input file');
end
if exist(fn,'file')==0
    error(['Input file : ' fn ' does not exist']);
end

if nargin < 2
    W=20;
end
if nargin < 3
    DO_LOESS=1;
end
if nargin <4
    MaxFunEvals=1e4;
end
if nargin <5
    MaxIter=5e3;
end

% convert string argin to numeric
W = str2double(W);
DO_LOESS= str2double(DO_LOESS);
MaxFunEvals= str2double(MaxFunEvals);
MaxIter= str2double(MaxIter);

fprintf('Loess window : %d\n',W);
fprintf('SkipLoess : %d\n',DO_LOESS);
fprintf('GMM Optimization MaxIter : %d\n',MaxIter);
fprintf('GMM Optimization MaxFunEvals : %d\n',MaxFunEvals);
%% import data
tic
fprintf('Loading file %s\n',fn) ;
nn = get_col_num(fn);
fid = fopen(fn);
C0 = textscan(fid,repmat('%s',1,nn),1,'delimiter','\t');
C = textscan(fid,repmat('%n',1,nn),'delimiter','\t');
fclose(fid);

chr=C{1};
pos=C{2};
sampleNames = cell(nn-2,1);
data=zeros(length(chr), nn-2);
header=cell(nn,1);
header{1}=C0{1}{1};
header{2}=C0{2}{1};

for i=3:nn
    sampleNames{i-2} = C0{i}{1};
    data(:,i-2)=C{i};
    header{i} = C0{i}{1};
end
clear fid C C0 i;
fprintf('\t\tdone in %0.3fsec\n',toc);
%% loess smoothing
if DO_LOESS == 1
    tic
    fprintf('Loess smoothing\n') ;
    chr_list=unique(chr);
    G=zeros(length(chr), nn-2);
    for i=1:length(chr_list)
        sel=chr==chr_list(i);
        fprintf('\tDoing chromosome %d\n\t\t', chr_list(i));
        for j=1:size(data,2)
            G(sel,j) = smooth(pos(sel),data(sel,j),W,'loess');
            if ~mod(i-1,100)
                fprintf('.');
            end
        end
    end
    fprintf('\n\t\tdone in %0.3fsec\n',toc);
    %% export loess smoothing
    tic
    fprintf('Exporting Loess-smoothed ratios\n') ;
    V=cell(nn,1);
    V{1}=chr;
    V{2}=pos;
    for i=1:nn-2
        V{i+2}=G(:,i);
    end

    % export data
    % pnum: 0=text, 1=real, 2=integer, 3=real max precision
    %   argin output, labs, V, num, overwrite
    print_report( [fn '.loess.dat'], header, V, [2 2 repmat(3,1,nn-2)]);

    fprintf('\n\t\tdone in %0.3fsec\n',toc);
else
    G=data;
end
clear data;
%% cnv calling

tic
fprintf('CNV calling\n') ;
[n1 n2] = size(G);
P = zeros(n1,n2);
for i=1:n1 % iterate all snps

    tmp = cnratio2cn(G(i,:), MaxFunEvals,MaxIter);
    % replace NaN by 0
    tmp(isnan(tmp) ) = 0;
    % get cnv dosage
    P(i,:) = tmp(:,1)+2*tmp(:,2)+3*tmp(:,3)+4*tmp(:,4);

    if ~mod(i-1,100)
        fprintf('.');
    end
end
fprintf('\n\t\tdone in %0.3fsec\n',toc);

%% export CNV calls
tic
fprintf('Exporting CNV dosage value\n') ;
V=cell(nn,1);
V{1}=chr;
V{2}=pos;
for i=1:nn-2
    V{i+2}=P(:,i);
end

% export data
% pnum: 0=text, 1=real, 2=integer, 3=real max precision
%   argin output, labs, V, num, overwrite
print_report( [fn '.cnv.dat'], header, V, [2 2 repmat(1,1,nn-2)]);

fprintf('\n\t\tdone in %0.3fsec\n',toc);

% subplot(2,1,1); plot(pos/1e6,G,'.'); subplot(2,1,2);plot(pos/1e6,P,'.')

